#!/usr/bin/perl -I/usr/local/sauron -w
#
#
#=================================
# project: Sauron managing
# script:  create_table_pef.pl
# version: 1.0\n";
#
# Vytvoreni tabulky pro PEF
#=================================


#--------------------------------------------------------------------------
# Nastaveni promennych
#--------------------------------------------------------------------------

#nastaveni ID zony, pro kterou se bude tabulka generovat
$PEFID = 6;

#nastaveni, zda zobrazovat take zaznamy bez HW adresy
$HW_ONLY = 1;

#--------------------------------------------------------------------------
# Ziskani dat pomoci SQL dotazu
#--------------------------------------------------------------------------

`mkdir -p TEMP`;

`psql -c "select hosts.id, hosts.grp, hosts.ether, a_entries.ip, hosts.domain from hosts, a_entries where ((hosts.zone = $PEFID) and (hosts.id = a_entries.host)) order by a_entries.ip" sauron > TEMP/id1.txt`;

`psql -c "select id, name from groups" sauron > TEMP/groups.txt`;
`psql -c "select host, grp from group_entries" sauron > TEMP/subgroups.txt`;
`psql -c "select ref, dhcp from dhcp_entries" sauron > TEMP/dhcp.txt`;

#--------------------------------------------------------------------------
# Nacteni dat
#--------------------------------------------------------------------------

#nacteni dat z tabulky host
open(ID, "<TEMP/id1.txt");
@data = <ID>;
close(ID);

#nacteni dat z tabulky groups
open(GRP, "<TEMP/groups.txt");
@grp = <GRP>;
close(GRP);

#nacteni dat z tabulky subgroups
open(SUBGRP, "<TEMP/subgroups.txt");
@subgrp = <SUBGRP>;
close(SUBGRP);

#nacteni dat z tabulky dhcp_entries
open(DHCPS, "<TEMP/dhcp.txt");
@dhcps = <DHCPS>;
close(DHCPS);

#--------------------------------------------------------------------------
# Predzpracovani nactenych dat
#--------------------------------------------------------------------------

@grp2 = ();
foreach(@grp){
  if (($_ =~ /^\s*[0-9][0-9]*/) == 1){
    #tj. pouze zaznamy
    push @grp2, "$_";
  }
}
@grp = @grp2;
@grp2 = ();

@subgrp2 = ();
#predzpracovani subgrp;
foreach(@subgrp){
  if (($_ =~ /^\s*[0-9][0-9]*/) == 1){
    #tj. pouze zaznamy
    push @subgrp2, "$_";
  }
}
@subgrp = @subgrp2;
@subgrp2 = ();

@dhcps2 = ();
#predzpracovani dhcps;
foreach(@dhcps){
  if (($_ =~ /^\s*[0-9][0-9]*/) == 1){
    #tj. pouze zaznamy
    push @dhcps2, "$_";
  }
}
@dhcps = @dhcps2;
@dhcps2 = ();


#--------------------------------------------------------------------------
# Zpracovani tabulky host
#--------------------------------------------------------------------------

@data2 = ();
foreach(@data){
  if (($_ =~ /^\s*[0-9][0-9]*/) == 1){
    #tj. pouze zaznamy

    #extrakce hosts.id
    $_ =~ /^\s*[0-9][0-9]*/;
    $id = $&;
    $rest = $';    
    $rest =~ /^\s*\|\s\s*/;
    $rest = $';

    #extrakce hosts.grp
    $rest =~ /^[^\s][^\s]*/;
    $grp_index = $&;
    $rest = $';
    $rest =~ /^\s*\|\s\s*/;
    $rest = $';

    #exttrakce hosts.ether
    $rest =~ /^[^\s\|][^\s\|]*/;
    $hw_address = $&;
    $rest = $';
    $rest =~ /^\s*\|\s\s*/;
    $rest = $';

    #extrakce IP (a_entries.ip)
    $rest =~ /^[^\s][^\s]*/;
    $ip = $&;
    $rest = $';
    $rest =~ /^\s*\|\s\s*/;
    $rest = $';

    #extrakce domenoveho jmena (hosts.domain)
    $rest =~ /^[^\s][^\s]*/;
    $domain = $&;
    $rest = $';

    #doupraveni ID (odstraneni mezer)
    $id =~ /[0-9][0-9]*/;
    $id = $&;

    #zjisteni jmena skupiny (groups.name)
    $gname = &get_grpname($grp_index);

    #zjisteni jmena bootovaciho souboru daneho hosta
    $filename = &get_filename($id);

    #zjisteni, zda se jedna o notebook
    $is_nb = &get_notebook($id);

    #pokud neni host clenem zadne skupiny, je misto nazvu skupiny uvedena pomlcka
    if ($gname eq "-1"){
      $gname = "-";
    }

    #pokud u zaznamu neni uvedena HW adresa je nahrazena pomlckou
    if ($hw_address =~ /\|/){
      $hw_address = "-           ";
    }

    #vygenerovani jedne radky tabulky - podle nastaveni $HW_ONLY mohou byt vynechany zaznamy
    #bez hw adresy
    if (($HW_ONLY == 0) | (($HW_ONLY == 1) && (($hw_address =~ /-/) == 0))){
      push @data2, "$hw_address\t$ip\t$domain\t$is_nb\t$gname\t$filename\n";
    }
  }
}

#--------------------------------------------------------------------------
# Vypis vysledku
#--------------------------------------------------------------------------
#
foreach(@data2){
  print $_;
}

#--------------------------------------------------------------------------
# Podprogramy
#--------------------------------------------------------------------------


sub get_grpname{
  $search=$_[0];
  foreach(@grp){
    $_ =~ /\|/;
    $loc_id = $`;
    $loc_ret = $';
    $loc_id =~ /[0-9][0-9]*/;
    $loc_id = $&;
    $loc_ret =~ /[^\s][^\s]*/;
    $loc_ret = $&;
    #print "$_ - > $loc_id =? $search\n";
    if ($loc_id == $search){
      return ($loc_ret);
    }
  }
  return("-1");
}


sub get_filename{
  $search=$_[0];
  foreach(@dhcps){
    $_ =~ /\|/;
    $loc_id = $`;
    $loc_ret = $';
    $loc_id =~ /[0-9][0-9]*/;
    $loc_id = $&;
    #print "$_ - > $loc_id =? $search\n";
    if ($loc_id == $search){
      if ($loc_ret =~ /^\s*filename\s*".*"/){
        $loc_pom = $&;
	$loc_pom =~ /^\s*filename\s*"/;
        $loc_pom = $';
	$loc_pom =~ /[^\s"][^\s"]*/;
        $loc_pom = $&;
        return ($loc_pom);
      }
    }
  }
  return("-");
}


sub get_notebook{
  $search=$_[0];
  $grp_id = -1;
  foreach(@subgrp){
    $_ =~ /\|/;
    $loc_id = $`;
    $loc_ret = $';
    $loc_id =~ /[0-9][0-9]*/;
    $loc_id = $&;
    $loc_ret =~ /[^\s][^\s]*/;
   $loc_ret = $&;
    #print "$_ - > $loc_id =? $search\n";
    if ($loc_id == $search){
      $grp_id = $loc_ret;
      $grp_loc_name = &get_grpname($grp_id);
      if ($grp_loc_name =~ /^notebook/){
        return("A");
      }
    }
  }
  return("N");
}
